@@ -3,11 +3,16 @@ import time
 import os
 import atexit
 import threading
+import subprocess
 from typing import Any
 
 from vertx import EventBus
 from queue import Queue
 
+class ServiceError(Exception):
+    def __init__(self, body:dict):
+        super().__init__(body.get("error"))
+        self.trace = body.get("trace")
 
 class BPMService:
     eb_calls: EventBus
@@ -15,7 +20,7 @@ class BPMService:
     delay = 10
     call_timeout = 30.0
     handlers = []
-    address_dict : dict
+    address_dict: dict
 
     def __init__(self, host="localhost", port=7000, options=None, err_handler=None, ssl_context=None):
         if options is None:
@@ -42,7 +47,7 @@ class BPMService:
         else:
             raise ValueError('No address supplied')
         tid = threading.get_ident()
-        self.address_dict[tid]=addr
+        self.address_dict[tid] = addr
 
     def getAddress(self, address: Any = None) -> str:
         if isinstance(address, dict):
@@ -67,14 +72,17 @@ class BPMService:
             body = {"reply": body}
         self.eb_calls.send(address=addr, headers=headers, body=body)
 
-    def fail(self, message:str=None, address=None, headers=None):
+    def fail(self, message: str = None, address=None, headers=None):
         addr = self.getAddress(address)
-        body = {"error": message}
+        body = {"error": message, "succeeded":False}
         self.eb_calls.send(address=addr, headers=headers, body=body)
 
     def call(self, address, body=None, headers=None):
         ret = self.send(address, headers, body)
         if 'body' in ret:
+            if 'succeeded' in ret['body']:
+                if not ret['succeeded']:
+                    raise ServiceError(ret['body'])
             if 'reply' in ret['body']:
                 return ret['body']['reply']
         return None
@@ -99,8 +107,16 @@ class BPMService:
             "variables": variables})
         return ret
 
+    def _run_external(self, address, handler):
+        my_env = os.environ.copy()
+        my_env["BPM_EVENT_BUS_REPLY"] = address["replyAddress"]
+        subprocess.Popen(sys.executable+" "+handler, env=my_env, shell=True)
+
     def register_handler(self, address, handler):
-        self.eb_handlers.register_handler(address, handler)
+        if isinstance(handler, str):
+            self.eb_handlers.register_handler(address, lambda msg: self._run_external(msg, handler))
+        else:
+            self.eb_handlers.register_handler(address, handler)
         self.handlers.append(address)
 
     def unregister_handler(self, address, handler=None):
